from formaloo import constants, client


class Tag:

    def __init__(self, title=None, description=None, slug=None):
        self.client = client.client
        self.title = title
        self.description = description
        self.slug = slug

    def get_body(self):
        body = {
            'title': self.title
        }

        if self.description:
            body['description'] = self.description

        if self.slug:
            body['slug'] = self.slug

        return body

    @staticmethod
    def get_list_body(tags):
        body = []

        for tag in tags:
            body.append(
                tag.get_body()
            )

        return body

    def get_list(self, **kwargs):
        params = kwargs

        response = self.client.get(
            constants.V_1_0_TAG_LIST_CREATE_ENDPOINT,
            params=params
        )

        return response.json()

    def create(self):
        if not self.title:
            raise ValueError("`title` is required to create a tag!")

        body = self.get_body()

        response = self.client.post(
            constants.V_1_0_TAG_LIST_CREATE_ENDPOINT,
            body=body
        )

        return response.json()

    def get(self, slug, **kwargs):
        params = kwargs

        response = self.client.get(
            constants.V_1_0_TAG_ITEM_ENDPOINT.format(slug),
            params=params
        )

        return response.json()



from .settings import V_1_0_API_BASE

# ENDPOINTS
V_1_0_AUTHORIZATION_TOKEN_ENDPOINT = '%soauth2/authorization-token/' % V_1_0_API_BASE

V_1_0_ACTIVITY_LIST_CREATE_ENDPOINT = '%sactivities/' % V_1_0_API_BASE
V_1_0_ACTIVITY_ITEM_ENDPOINT = '%sactivities/{}/' % V_1_0_API_BASE
V_1_0_ACTIVITY_BATCH_ENDPOINT = '%sactivities/batch/' % V_1_0_API_BASE

V_1_0_CUSTOMER_LIST_CREATE_ENDPOINT = '%scustomers/' % V_1_0_API_BASE
V_1_0_CUSTOMER_ITEM_ENDPOINT = '%scustomers/{}/' % V_1_0_API_BASE
V_1_0_CUSTOMER_BATCH_ENDPOINT = '%scustomers/batch/' % V_1_0_API_BASE

V_1_0_TAG_LIST_CREATE_ENDPOINT = '%stags/' % V_1_0_API_BASE
V_1_0_TAG_ITEM_ENDPOINT = '%stags/{}/' % V_1_0_API_BASE

V_1_0_CREATE_GAMIFICATION_CALCULATION_JOB = '%sgamification-calculation-jobs/' % V_1_0_API_BASE

# HEADERS
APPLICATION_HEADER = 'x-api-key'
AUTHORIZATION_HEADER = 'Authorization'
AUTHORIZATION_BEARER = 'JWT'
CREDENTIAL_BEARER = 'Basic'
CREDENTIAL_GRAT_TYPE = 'client_credentials'

# Settings
AUTHORIZATION_TOKEN_TIMEOUT = 5*60

# Helpers
CUSTOMER_BASE_FIELDS = [
    'first_name',
    'last_name',
    'full_name',
    'email',
    'phone_number',
    'score',
    'city',
    'language',
    'created_at',
    'code',
    'username'
]



from formaloo import constants, client


class CustomerBatch:

    def __init__(self, customers=None):
        if not customers:
            customers = []

        self.client = client.client
        self.customers = customers

    def add_customer(self, customer):
        self.customers.append(customer)

    def create(self):
        body = self.get_body()

        response = self.client.post(
            constants.V_1_0_CUSTOMER_BATCH_ENDPOINT,
            body=body
        )

        return response.json()

    def get_body(self):
        body = {
            'customers_data': []
        }

        for customer in self.customers:
            body['customers_data'].append(
                customer.get_body()
            )

        return body



from formaloo import constants, client


class GamificationCalculationJob:
    def __init__(self):
        self.client = client.client

    def create(self):
        body = self.get_body()

        response = self.client.post(
            constants.V_1_0_CREATE_GAMIFICATION_CALCULATION_JOB,
            body=body
        )

        return response.json()

    def get_body(self):
        body = {
        }
        return body


from datetime import timedelta, datetime
import json

from requests.models import Response
import requests

from . import constants
from .settings import CLIENT_SECRET, CLIENT_KEY


class Client:
    def __init__(self):
        self.authorization_token = None
        self.AUTHORIZATION_EXPIRY_TIME = None

    def _get_headers(self, content_type='application/json', include_auth_header=True):
        if content_type:
            headers = {
                'Content-type': content_type
            }
        else:
            headers = {}

        headers.update(self._get_application_header())

        if include_auth_header:
            headers.update(self._get_authorization_header())

        return headers

    def _get_application_header(self):
        headers = {
        }

        if CLIENT_KEY:
            headers[constants.APPLICATION_HEADER] = CLIENT_KEY

        return headers

    def _get_authorization_header(self):
        headers = {}

        if CLIENT_SECRET:
            if (
                not self.authorization_token or
                self.authorization_expiry_time < datetime.now()
            ):
                self._obtain_authorization_token()

            headers[constants.AUTHORIZATION_HEADER] = "{bearer} {token}".format(
                bearer=constants.AUTHORIZATION_BEARER,
                token=self.authorization_token
            )

        return headers

    def _obtain_authorization_token(self):
        if CLIENT_SECRET:
            request_headers = self._get_application_header()

            request_headers[constants.AUTHORIZATION_HEADER] = "{bearer} {token}".format(
                bearer=constants.CREDENTIAL_BEARER,
                token=CLIENT_SECRET
            )

            request_body = {
                'grant_type': constants.CREDENTIAL_GRAT_TYPE
            }

            response = requests.post(
                constants.V_1_0_AUTHORIZATION_TOKEN_ENDPOINT,
                headers=request_headers,
                data=request_body
            )

            if response.status_code == 200:
                self.authorization_token = response.json().get('authorization_token')
                self.authorization_expiry_time = datetime.now() + timedelta(
                    seconds=constants.AUTHORIZATION_TOKEN_TIMEOUT
                )

            elif response.status_code in [400, 401, 403]:
                try:
                    # TODO handle errors properly
                    raise Exception(response.json().get('errors'))
                except:
                    raise Exception(
                        "Unknown error with code {}".format(
                            response.status_code
                        )
                    )

    def get_blank_response(self):
        response = Response()
        response._content = json.dumps({}).encode('utf-8')
        response.encoding = 'utf-8'
        response.status_code = 204
        return response

    def post(self, endpoint, body, include_auth_header=True, customer_headers={}):
        headers = self._get_headers(
            include_auth_header=include_auth_header
        )

        # If user has set key and secret to and empty value, don't send request. (Used for test purposes)
        if not constants.APPLICATION_HEADER in headers:
            return self.get_blank_response()

        response = requests.post(
            url=endpoint,
            headers=headers,
            json=body
        )

        return response

    def get(self, endpoint, params={}, include_auth_header=True, customer_headers={}):
        headers = self._get_headers(
            include_auth_header=include_auth_header
        )

        # If user has set key and secret to and empty value, don't send request. (Used for test purposes)
        if not constants.APPLICATION_HEADER in headers:
            return self.get_blank_response()

        response = requests.get(
            url=endpoint,
            headers=headers,
            params=params
        )

        return response


client = Client()


from formaloo import constants, client
from formaloo.tags import Tag


class Customer:

    def __init__(self, base_data=None, extra_data=None, tags=None, **kwargs):
        if not base_data:
            base_data = {}

        if not extra_data:
            extra_data = {}

        if not tags:
            tags = []

        self.client = client.client
        self.base_customer_data = base_data
        self.extra_customer_data = extra_data
        self.tags = tags

        for key, value in kwargs.items():
            if key in constants.CUSTOMER_BASE_FIELDS:
                self.base_customer_data[key] = value
            else:
                self.extra_customer_data[key] = value

    def get_body(self):
        tags_body = Tag.get_list_body(self.tags)

        body = {
            'customer_data': self.extra_customer_data,
            'tags': tags_body
        }
        body.update(self.base_customer_data)

        return body

    def get_list(self, **kwargs):
        params = kwargs

        response = self.client.get(
            constants.V_1_0_CUSTOMER_LIST_CREATE_ENDPOINT,
            params=params
        )

        return response.json()

    def create(self):
        body = self.get_body()

        response = self.client.post(
            constants.V_1_0_CUSTOMER_LIST_CREATE_ENDPOINT,
            body=body
        )

        return response.json()

    def get(self, code, **kwargs):
        params = kwargs

        response = self.client.get(
            constants.V_1_0_CUSTOMER_ITEM_ENDPOINT.format(code),
            params=params
        )

        return response.json()



from environs import Env

env = Env()
env.read_env()

try:
    from django.conf import settings

    V_1_0_API_BASE = getattr(settings, "FORMALOO_API_BASE", None)
    CLIENT_KEY = getattr(settings, 'FORMALOO_CLIENT_KEY', None)
    CLIENT_SECRET = getattr(settings, 'FORMALOO_CLIENT_SECRET', None)

except ModuleNotFoundError:
    V_1_0_API_BASE = None
    CLIENT_KEY = None
    CLIENT_SECRET = None


if V_1_0_API_BASE == None:
    V_1_0_API_BASE = env("FORMALOO_API_BASE", 'https://api.formaloo.net/v1.0/')

if CLIENT_KEY == None:
    CLIENT_KEY = env('FORMALOO_CLIENT_KEY')

if CLIENT_SECRET == None:
    CLIENT_SECRET = env('FORMALOO_CLIENT_SECRET', None)



from formaloo import constants, client
from formaloo.tags import Tag


class Activity:

    def __init__(self, action=None, customer_data=None, activity_data=None, activity_date=None, tags=None):
        if not activity_data:
            activity_data = {}

        if not tags:
            tags = []

        if not customer_data:
            customer_data = {}

        self.client = client.client
        self.action = action
        self.customer_data = customer_data
        self.activity_data = activity_data
        self.activity_date = activity_date
        self.tags = tags

    def get_body(self):
        tags_body = Tag.get_list_body(self.tags)

        body = {
            'action': self.action,
            'customer': self.customer_data,
            'activity_data': self.activity_data,
            'activity_date': self.activity_date,
            'tags': tags_body
        }

        return body

    def get_list(self, **kwargs):
        params = kwargs

        response = self.client.get(
            constants.V_1_0_ACTIVITY_LIST_CREATE_ENDPOINT,
            params=params
        )

        return response.json()

    def create(self):
        if not self.action:
            raise ValueError("`action` is required to create a tag!")

        body = self.get_body()

        response = self.client.post(
            constants.V_1_0_ACTIVITY_LIST_CREATE_ENDPOINT,
            body=body
        )

        return response.json()

    def get(self, slug, **kwargs):
        params = kwargs

        response = self.client.get(
            constants.V_1_0_ACTIVITY_ITEM_ENDPOINT,
            params=params
        )

        return response.json()



import os
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


def strip_comments(l):
    return l.split('#', 1)[0].strip()


def reqs(*f):
    return [
        r for r in (
            strip_comments(l) for l in open(
                os.path.join(os.getcwd(), 'requirements', *f)).readlines()
        ) if r]


setuptools.setup(
    name="formaloo-cdp",
    version="0.2.3",
    author="Formaloo",
    author_email="info@formaloo.com",
    description="Official SDK to use Formaloo API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/formaloo/formaloo-python",
    project_urls={
        "Bug Tracker": "https://github.com/formaloo/formaloo-python/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(exclude=['t', 't.*']),
    python_requires=">=3.6",
    install_requires=reqs('default.txt'),
)



from formaloo import constants, client


class ActivityBatch:

    def __init__(self, activities=[]):
        self.client = client.client
        self.activities = activities

    def add_activity(self, activity):
        self.activities.append(activity)

    def create(self):
        body = self.get_body()

        response = self.client.post(
            constants.V_1_0_ACTIVITY_BATCH_ENDPOINT,
            body=body
        )

        return response.json()

    def get_body(self):
        body = {
            'activities_data': []
        }

        for activity in self.activities:
            body['activities_data'].append(
                activity.get_body()
            )

        return body
